//
//  CubeViewShader.m
//  Cube
//
//  Created by Chris Osborn on 7/1/15.
//  Copyright (c) 2015 Google. All rights reserved.
//

#import "CubeViewShader.h"
#import <Performer/PMRMacros.h>

static NSString *vertexShaderSource = PMRStringValue
(
 precision highp float;
 
 attribute vec4 a_position;
 attribute vec3 a_normal;
 varying vec3 v_normal;
 uniform mat4 u_transformProjectionMatrix;
 uniform mat3 u_normalMatrix;
 uniform vec3 u_size;
 
 void main() {
     gl_Position = u_transformProjectionMatrix * vec4(a_position.xyz * u_size,
                                                      a_position.w);
     v_normal = u_normalMatrix * a_normal;
 }
);

static NSString *fragmentShaderSource = PMRStringValue
(
 precision mediump float;
 
 varying vec3 v_normal;
 uniform vec4 u_color;
 uniform vec3 u_lightDirection;
 
 const float kAmbient = 0.2;
 
 void main() {
     vec3 n = normalize(v_normal);
     vec3 l = normalize(u_lightDirection);
     float diffuse = clamp(dot(n, l), 0.0, 1.0);
     gl_FragColor = vec4(u_color.rgb * (diffuse + kAmbient), u_color.a);
 }
);

NSString *const CubeViewShaderKey = @"CubeViewShaderKey";

@implementation CubeViewShader {
    GLint _normalMatrixUniform;
    GLint _sizeUniform;
    GLint _colorUniform;
    GLint _lightDirectionUniform;
}

- (instancetype)init {
    if ((self = [super initWithVertexShaderSource:vertexShaderSource
                             fragmentShaderSource:fragmentShaderSource]) != nil) {
        [self bindAttribute:RIShaderAttributeNamePosition toIndex:RIShaderAttributePosition];
        [self bindAttribute:RIShaderAttributeNameNormal toIndex:RIShaderAttributeNormal];
        [self link];
        
        _normalMatrixUniform = [self uniformLocation:@"u_normalMatrix"];
        _sizeUniform = [self uniformLocation:@"u_size"];
        _colorUniform = [self uniformLocation:@"u_color"];
        _lightDirectionUniform = [self uniformLocation:@"u_lightDirection"];
    }
    return self;
}

- (void)setNormalMatrix:(GLKMatrix3)normalMatrix {
    _normalMatrix = normalMatrix;
    
    if (_normalMatrixUniform >= 0) {
        GLint normalMatrixUniform = _normalMatrixUniform;
        [[RIRenderContext activeContext] dispatchBlock:^{
            glUniformMatrix3fv(normalMatrixUniform,
                               1, 0, normalMatrix.m);
        }];
    }
}

- (void)setSize:(RISize3F)size {
    if (RISizeIsEqualToSize3F(size, _size))
        return;
    
    _size = size;
    
    if (_sizeUniform >= 0) {
        GLint sizeUniform = _sizeUniform;
        [[RIRenderContext activeContext] dispatchBlock:^{
            glUniform3f(sizeUniform,
                        size.width, size.height, size.depth);
        }];
    }
}

- (void)setColor:(RIColorRGBA)color {
    if (RIColorIsEqualToColorRGBA(color, _color))
        return;
    
    _color = color;
    
    if (_colorUniform >= 0) {
        GLint colorUniform = _colorUniform;
        [[RIRenderContext activeContext] dispatchBlock:^{
            glUniform4f(colorUniform,
                        color.red, color.green, color.blue, color.alpha);
        }];
    }
}

- (void)setLightDirection:(RIPoint3F)lightDirection {
    lightDirection = RIPointNormalize3F(lightDirection);
    if (RIPointIsEqualToPoint3F(lightDirection, _lightDirection))
        return;
    
    _lightDirection = lightDirection;
    
    if (_lightDirectionUniform >= 0) {
        GLint lightDirectionUniform = _lightDirectionUniform;
        [[RIRenderContext activeContext] dispatchBlock:^{
            glUniform3f(lightDirectionUniform,
                        lightDirection.x, lightDirection.y, lightDirection.z);
        }];
    }
}

@end
